unit STDBOX;

{$O+}
{$F+}

interface

uses objects,msgbox,drivers,dialogs,xbeep,views,App;


{ Ligne de saisie numrique - Renvoi une chaine de caractres
    Min  : Valeur minimum
    Max  : Valeur maximum
    Mess : Message affich en cas d'erreur

}

type PNumInputLine = ^ TNumInputLine;
     TNumInputLine = object(TInputLine)
       VMax,VMin:Word;
       ErrMess:String;

       constructor init(var Bounds: TRect; AMaxLen: Integer; Min,Max:Word; Mess:String);
       function valid(Cm:word):boolean; virtual;
       procedure HandleEvent(var Event:Tevent); virtual;
     end;


{ Ligne de saisie secrete - Renvoi une chaine de caractres }

type PSecretInput = ^ TSecretInput;
     TSecretInput = object(TInputLine)
       SecretChar:Char;

       constructor Init(var Bounds: TRect; AMaxLen: Integer; Secret:char);
       procedure HandleEvent(var E:TEvent); virtual;
       procedure Draw; virtual;
     end;



{ ListBox gnrant un evbroadcast : cmitemfocused  chaque slection }

type PFocusListBox = ^TFocusListBox;
     TFocusListBox = object(TListBox)
       procedure FocusItem(Item:integer); virtual;
     end;


{ Boite de dialogue avec OK automatique (par RETURN) }

type PAutoDialog = ^TAutoDialog;
     TAutoDialog = object(TDialog)
       procedure HandleEvent(var E:TEvent); virtual;
       procedure sizelimits(var Min,Max:Tpoint); virtual;
     end;


procedure LabelInputBox(var Bounds:TRect; Line:PView; ILabel:PLabel; HC:word; var Result);
  { Creer une boite de saisie avec:
       Bounds:Cadre de la boite
       Line:Ligne de saisie ou autre.
       Label:Label de la ligne de saisie
       HC:Help context
       Result:Chaine de resultat
  }


{ Execution d'une boite de dialogue - Retour = cmxxx }

function ExecDialog(P: PDialog; Data: Pointer): Word;

{ test }

type PSCollection = ^TSCollection;
TSCollection = object(TSortedCollection)
    function Compare(Key1, Key2: Pointer): Integer; virtual;
    procedure FreeItem(Item: Pointer); virtual;
    end;


const cmItemFocused = 1100;

implementation

procedure LabelInputBox(var Bounds:TRect; Line:PView; ILabel:PLabel; HC:word; var Result);

var Dialog:PAutoDialog;
    Control:word;

begin
Dialog:=New(PAutoDialog,Init(Bounds,''));
with Dialog^ do
  begin
  insert(Line);
  insert(ILabel);
  SetData(Result);
  Options:=Options or ofCentered;
  helpctx:=HC;
  end;

Control:=DeskTop^.ExecView(Dialog);

if Control<>cmcancel then Dialog^.GetData(Result);
Dispose(Dialog,Done);
end;


function ExecDialog(P: PDialog; Data: Pointer): Word;
var
  Result: Word;
begin
  Result := cmCancel;
  P := PDialog(Application^.ValidView(P));
  if P <> nil then
  begin
    if Data <> nil then P^.SetData(Data^);
    Result := DeskTop^.ExecView(P);
    if (Result <> cmCancel) and (Data <> nil) then P^.GetData(Data^);
    Dispose(P, Done);
  end;
  ExecDialog := Result;
end;


constructor TNumInputLine.init(var Bounds: TRect; AMaxLen: Integer; Min,Max:Word; Mess:String);

begin
TInputLine.init(Bounds, AMaxLen);
VMin:=Min;
VMax:=Max;
ErrMess:=Mess;
end;

procedure TNumInputLine.HandleEvent(var Event:TEvent);

begin

if (Event.what<>evkeydown) or (not (Event.CharCode in ['A'..'Z','a'..'z',',']))
   then TInputLine.HandleEvent(Event);

end;

function TNumInputLine.Valid(cm:word):boolean;

 var Ok:boolean;
     E:integer;
     V:word;
     R:TRect;

begin
  Ok:=TInputLine.Valid(cm);

  R.assign(20,5,60,12);

  val(Data^,V,E);
  if (E<>0) or (V<VMin) or (V>VMax) then
   begin
   Beep;
   MessageBoxRect(R,ErrMess,
    nil, mfError + mfOkButton);
   Ok:=false;
   end;

  Valid:=Ok;
end;



constructor TSecretInput.Init(var Bounds: TRect; AMaxLen: Integer; Secret:char);

begin
TInputLine.Init(Bounds,AMaxLen);
SecretChar:=Secret;
end;

procedure TSecretInput.HandleEvent(var E:TEvent);

begin
if (E.what=evkeydown) and (E.Keycode=kbenter) then
   begin
   E.what:=evcommand;
   E.command:=cmok;
   end;
TInputLine.HandleEvent(E);
end;


procedure TSecretInput.Draw;

var P:PString;
    S:String;
    f:byte;

begin
P:=Data;
S:='';
for f:=1 to length(P^) do S:=S+SecretChar;
Data:=@S;
TInputLine.Draw;
Data:=P;
end;

procedure TAutoDialog.HandleEvent(var E:TEvent);

begin
if (E.what=evkeydown) and (E.Keycode=kbenter) then
   begin
   E.what:=evcommand;
   E.command:=cmok;
   end;
TDialog.HandleEvent(E);
end;

procedure TAutoDialog. sizelimits(var Min,Max:Tpoint);

begin
TDialog.SizeLimits(Min,Max);
Min.X:=2;
Min.Y:=2;
end;


procedure TFocusListBox.FocusItem(Item:integer);

begin
TListBox.FocusItem(Item);
Message(Owner, evBroadcast, cmItemFocused, List^.At(Item));
end;

function TSCollection.Compare(Key1, Key2: Pointer): Integer;

begin
if PString(key1)^<Pstring(key2)^ then compare:=-1;
if PString(key1)^=Pstring(key2)^ then compare:=0;
if PString(key1)^>Pstring(key2)^ then compare:=1;
end;

procedure TSCollection.FreeItem(Item:Pointer);

begin
disposestr(Item);
end;


end.